library(tidyverse)
library(brms) 
library(tidybayes)  
library(brmsmargins) 
library(gt)
library(glue)
library(ggplot2)


options(mc.cores = 2, brms.backend = 'cmdstanr', brms.file_refit = 'on_change')

# Set ggplot theme
theme_set(
  theme_bw() + theme(panel.grid = element_blank(), strip.background = element_blank())
)

getwd()

all_IPRL <- read.csv(file="All_IPRL.csv", header=TRUE, sep=",") %>%
  rename_with(function(x) gsub('[[:punct:]]', '_', x)) %>%
  rename_with(function(x) gsub(' ', '_', x)) %>%
  mutate(fertilizer = factor(fertilizer, levels = c('L','M','H')),
         plant = factor(plant),
         pre_vs_post = factor(pre_vs_post, levels = c('pre', 'post')))


month_offset <- 21

all_IPRL <- all_IPRL %>%
  mutate(months_since_exp_start = month + 12*(year-17) - 10,
         months_since_trt_start = pmax(0, months_since_exp_start - month_offset))



##########LINEAR TIP DENSITY#############

ggplot(all_IPRL, aes(x = months_since_exp_start, y = tip_density_crown_diam, color = fertilizer)) +
  geom_point(alpha = 1/4) +
  geom_boxplot(aes(group = interaction(fertilizer, months_since_exp_start))) +
  stat_summary(geom = 'line', fun = median) +
  geom_vline(xintercept = month_offset, size = 1.5) +
  scale_y_log10()



logtipdensity_monthspline_trt_fit <- brm(
  log(tip_density_crown_diam) ~ fertilizer + pre_vs_post + months_since_exp_start + months_since_trt_start + fertilizer:pre_vs_post + fertilizer:months_since_exp_start + fertilizer:months_since_trt_start + s(month, k = 10, m = 1) + (1 | plant), 
  data = all_IPRL, family = gaussian, 
  prior = c(
    prior(normal(0, 3), class = b)
  ),
  chains = 4, iter = 3000, warmup = 2000, seed = 2287,
  file = '/Users/dalehalbritter/Documents/Demography/logtipdensity_monthspline_trt'
)

max(rhat(logtipdensity_monthspline_trt_fit))
 
pp_check(logtipdensity_monthspline_trt_fit)

pred_grid_real <- expand_grid(fertilizer = unique(all_IPRL$fertilizer), months_since_exp_start = 0:42) %>%
  mutate(pre_vs_post = factor(months_since_exp_start > month_offset, labels = c('pre', 'post')),
         months_since_trt_start = pmax(0, months_since_exp_start - month_offset),
         month = (months_since_exp_start + 9) %% 12 + 1)

pred_grid_counterfactual <- pred_grid_real %>%
  filter(pre_vs_post == 'post') %>%
  mutate(pre_vs_post = factor('pre', levels = c('pre', 'post')), months_since_trt_start = 0)


logtipdensity_predictions_real <- pred_grid_real %>%
  add_epred_draws(logtipdensity_monthspline_trt_fit, re_formula = ~ month) %>%
  mutate(scenario = 'real')

logtipdensity_predictions_counterfactual <- pred_grid_counterfactual %>%
  add_epred_draws(logtipdensity_monthspline_trt_fit, re_formula = ~ month) %>%
  mutate(scenario = 'counterfactual')

logtipdensity_predictions <- bind_rows(logtipdensity_predictions_real, logtipdensity_predictions_counterfactual) %>%
  mutate(.epred = exp(.epred))



fert_label <- labeller(fertilizer = c(L = 'low fertilizer', M = 'medium fertilizer', H = 'high fertilizer'))

ggplot(logtipdensity_predictions, aes(x = months_since_exp_start, y = .epred, fill = scenario)) +
  facet_wrap(~ fertilizer, nrow = 3, labeller = fert_label) +
  stat_lineribbon(.width = c(0.95, 0.66), alpha = 1/2) +
  labs(y = 'Tips per cm crown diameter (modeled)') +
  scale_x_continuous(name = 'Months since start of experiment', expand = c(0, 0)) +
  theme(legend.position = c(0.1, 0.9), legend.background = element_blank()) +
  scale_fill_brewer(palette = 'Dark2')




ltd_pred_real <- logtipdensity_predictions_real %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_real = .epred)

ltd_pred_cf <- logtipdensity_predictions_counterfactual %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_cf = .epred)

ltd_pred_wide <- inner_join(ltd_pred_real, ltd_pred_cf) %>%
  mutate(contrast = exp(pred_real - pred_cf), month = months_since_exp_start - month_offset) 



ggplot(ltd_pred_wide, aes(x = month, y = contrast)) + 
  facet_wrap(~ fertilizer, nrow = 3, labeller = fert_label) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  labs(y = 'Ratio of tip density with thrips/without thrips (modeled)') +
  geom_hline(yintercept = 1, linetype = 'dashed', color = 'gray50') +
  ggtitle('Inferred treatment effect on tip density (IPRL)', 'by fertilizer level at different times since release')


ltd_pred_wider <- ltd_pred_wide %>%
  select(-c(months_since_exp_start)) %>%
  pivot_wider(id_cols = c(.chain, .iteration, .draw, month), names_from = fertilizer, values_from = contrast) %>%
  mutate(contr_HL = H - L, contr_HM = H - M, contr_ML = M - L) %>%
  select(-c(L,M,H)) %>%
  pivot_longer(starts_with('contr'), values_to = 'contrast')

ggplot(ltd_pred_wider, aes(x = month, y = contrast)) +
  facet_wrap(~ name, nrow = 3, labeller = labeller(name = c(contr_HL = 'high - low', contr_HM = 'high - medium', contr_ML = 'medium - low'))) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  labs(y = 'Difference in inferred treatment effect (modeled)') +
  geom_hline(yintercept = 0, linetype = 'dashed', color = 'gray50') +
  ggtitle('Difference in inferred treatment effect on tip density (IPRL)', 'between pairs of fertilizer levels at different times since release')




###########Proportion of dead tips##############

ggplot(all_IPRL, aes(x = months_since_exp_start, y = percent_dead, color = fertilizer)) +
  geom_point(alpha = 1/4) +
  geom_boxplot(aes(group = interaction(fertilizer, months_since_exp_start))) +
  stat_summary(geom = 'line', fun = median) +
  geom_vline(xintercept = month_offset, size = 1.5) 


propdeadtips_monthspline_trt_fit <- brm(
  dead_tips | trials(total_tips) ~ fertilizer + pre_vs_post + months_since_exp_start + months_since_trt_start + fertilizer:pre_vs_post + fertilizer:months_since_exp_start + fertilizer:months_since_trt_start + s(month, k = 10, m = 1) + (1 | plant), 
  data = all_IPRL, family = binomial(link = 'logit'), 
  prior = c(
    prior(normal(0, 3), class = b)
  ),
  chains = 4, iter = 3000, warmup = 2000, seed = 2288,
  file = '/Users/dalehalbritter/Documents/Demography/propdeadtips_monthspline_trt'
)


max(rhat(propdeadtips_monthspline_trt_fit))

pp_check(propdeadtips_monthspline_trt_fit) + scale_x_log10()



propdeadtips_predictions_real <- pred_grid_real %>%
  mutate(total_tips = 1) %>%
  add_epred_draws(propdeadtips_monthspline_trt_fit, re_formula = ~ month) %>%
  mutate(scenario = 'real')

propdeadtips_predictions_counterfactual <- pred_grid_counterfactual %>%
  mutate(total_tips = 1) %>%
  add_epred_draws(propdeadtips_monthspline_trt_fit, re_formula = ~ month) %>%
  mutate(scenario = 'counterfactual')

propdeadtips_predictions <- bind_rows(propdeadtips_predictions_real, propdeadtips_predictions_counterfactual) 



ggplot(propdeadtips_predictions, aes(x = months_since_exp_start, y = .epred, fill = scenario)) +
  facet_wrap(~ fertilizer, nrow = 3, labeller = labeller(fertilizer = c(L = 'low fertilizer', M = 'medium fertilizer', H = 'high fertilizer'))) +
  stat_lineribbon(.width = c(0.95, 0.66), alpha = 1/2) +
  labs(y = 'Proportion dead tips (modeled)') +
  ylim(0,1) +
  scale_x_continuous(name = 'Months since start of experiment', expand = c(0, 0)) +
  theme(legend.position = c(0.1, 0.9), legend.background = element_blank()) +
  scale_fill_brewer(palette = 'Dark2')


pdt_pred_real <- propdeadtips_predictions_real %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_real = .epred)

pdt_pred_cf <- propdeadtips_predictions_counterfactual %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_cf = .epred)

pdt_pred_wide <- inner_join(pdt_pred_real, pdt_pred_cf) %>%
  mutate(contrast = pred_real/pred_cf, month = months_since_exp_start - month_offset) 


ggplot(pdt_pred_wide, aes(x = month, y = contrast)) + 
  facet_wrap(~ fertilizer, nrow = 3, labeller = fert_label) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  labs(y = 'Ratio of proportion dead tips with thrips/without thrips (modeled)') +
  geom_hline(yintercept = 1, linetype = 'dashed', color = 'gray50') +
  ggtitle('Inferred treatment effect on proportion dead tips (IPRL)', 'by fertilizer level at different times since release')



pdt_pred_wider <- pdt_pred_wide %>%
  select(-c(months_since_exp_start)) %>%
  pivot_wider(id_cols = c(.chain, .iteration, .draw, month), names_from = fertilizer, values_from = contrast) %>%
  mutate(contr_HL = H - L, contr_HM = H - M, contr_ML = M - L) %>%
  select(-c(L,M,H)) %>%
  pivot_longer(starts_with('contr'), values_to = 'contrast')

ggplot(pdt_pred_wider, aes(x = month, y = contrast)) +
  facet_wrap(~ name, nrow = 3, labeller = labeller(name = c(contr_HL = 'high - low', contr_HM = 'high - medium', contr_ML = 'medium - low'))) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  labs(y = 'Difference in inferred treatment effect (modeled)') +
  geom_hline(yintercept = 0, linetype = 'dashed', color = 'gray50') +
  ggtitle('Difference in inferred treatment effect on proportion dead tips (IPRL)', 'between pairs of fertilizer levels at different times since release')

