library(tidyverse)
library(brms) 
library(tidybayes)  
library(brmsmargins) 
library(gt)
library(glue)
library(ggplot2)


options(mc.cores = 2, brms.backend = 'cmdstanr', brms.file_refit = 'on_change')

# Set ggplot theme
theme_set(
  theme_bw() + theme(panel.grid = element_blank(), strip.background = element_blank())
)

getwd()

#IRREC = UF in the manuscript. It was originally coded as IRREC but changed to UF for brevity in the manuscript
all_IRREC <- read.csv(file="All_UF.csv", header=TRUE, sep=",") %>%
  rename_with(function(x) gsub('[[:punct:]]', '_', x)) %>%
  rename_with(function(x) gsub(' ', '_', x)) %>%
  mutate(fertilizer = factor(fertilizer, levels = c('L','M','H')),
         plant = factor(plant),
         pre_vs_post = factor(pre_vs_post, levels = c('pre', 'post')))


month_offset2 <- 20

all_IRREC <- all_IRREC %>%
  mutate(months_since_exp_start = month + 12*(year-17) - 12,
         months_since_trt_start = pmax(0, months_since_exp_start - month_offset2))



##########LINEAR TIP DENSITY#############

ggplot(all_IRREC, aes(x = months_since_exp_start, y = tip_density_crown_diam, color = fertilizer)) +
  geom_point(alpha = 1/4) +
  geom_boxplot(aes(group = interaction(fertilizer, months_since_exp_start))) +
  stat_summary(geom = 'line', fun = median) +
  geom_vline(xintercept = month_offset, size = 1.5) +
  scale_y_log10()



logtipdensity_monthspline_trt_fit2 <- brm(
  log(tip_density_crown_diam) ~ fertilizer + pre_vs_post + months_since_exp_start 
  + months_since_trt_start + fertilizer:pre_vs_post + fertilizer:months_since_exp_start 
  + fertilizer:months_since_trt_start 
  + s(month, k = 10, m = 1) + (1 | plant), 
  data = all_IRREC, family = gaussian, 
  prior = c(
    prior(normal(0, 3), class = b)
  ),
  chains = 4, iter = 3000, warmup = 2000, seed = 2287,
  file = '/Users/dalehalbritter/Documents/Demography/logtipdensity_monthspline_trt2'
)

max(rhat(logtipdensity_monthspline_trt_fit2))

pp_check(logtipdensity_monthspline_trt_fit2)

pred_grid_real2 <- expand_grid(fertilizer = unique(all_IRREC$fertilizer), months_since_exp_start = 0:33) %>%
  mutate(pre_vs_post = factor(months_since_exp_start > month_offset2, labels = c('pre', 'post')),
         months_since_trt_start = pmax(0, months_since_exp_start - month_offset2),
         month = (months_since_exp_start + 11) %% 12 + 1)

pred_grid_counterfactual2 <- pred_grid_real2 %>%
  filter(pre_vs_post == 'post') %>%
  mutate(pre_vs_post = factor('pre', levels = c('pre', 'post')), months_since_trt_start = 0)


logtipdensity_predictions_real2 <- pred_grid_real2 %>%
  add_epred_draws(logtipdensity_monthspline_trt_fit2, re_formula = ~ month) %>%
  mutate(scenario = 'real')

logtipdensity_predictions_counterfactual2 <- pred_grid_counterfactual2 %>%
  add_epred_draws(logtipdensity_monthspline_trt_fit2, re_formula = ~ month) %>%
  mutate(scenario = 'counterfactual')

logtipdensity_predictions2 <- bind_rows(logtipdensity_predictions_real2, logtipdensity_predictions_counterfactual2) %>%
  mutate(.epred = exp(.epred))



fert_label <- labeller(fertilizer = c(L = 'low fertilizer', M = 'medium fertilizer', H = 'high fertilizer'))

ggplot(logtipdensity_predictions2, aes(x = months_since_exp_start, y = .epred, fill = scenario)) +
  facet_wrap(~ fertilizer, nrow = 3, labeller = fert_label) +
  stat_lineribbon(.width = c(0.95, 0.66), alpha = 1/2) +
  labs(y = 'Tips per cm crown diameter (modeled)') +
  ylim(0,3) +
  scale_x_continuous(name = 'Months since start of experiment', expand = c(0, 0)) +
  theme(legend.position = c(0.1, 0.9), legend.background = element_blank()) +
  scale_fill_brewer(palette = 'Dark2')



ltd_pred_real2 <- logtipdensity_predictions_real2 %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_real = .epred)

ltd_pred_cf2<- logtipdensity_predictions_counterfactual2 %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_cf = .epred)

ltd_pred_wide2 <- inner_join(ltd_pred_real2, ltd_pred_cf2) %>%
  mutate(contrast = exp(pred_real - pred_cf), month = months_since_exp_start - month_offset) 

ltd_pred_wide_nofert <- inner_join(ltd_pred_real2, ltd_pred_cf2) %>%
  mutate(contrast = exp(pred_real - pred_cf), month = months_since_exp_start - month_offset) 

ggplot(ltd_pred_wide2, aes(x = month, y = contrast)) + 
  facet_wrap(~ fertilizer, nrow = 3, labeller = fert_label) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  labs(y = 'Ratio of tip density with thrips/without thrips (modeled)') +
  geom_hline(yintercept = 1, linetype = 'dashed', color = 'gray50') +
  ggtitle('Inferred treatment effect on tip density (IRREC)', 'by fertilizer level at different times since release')

ltd_pred_wider2 <- ltd_pred_wide2 %>%
  select(-c(months_since_exp_start)) %>%
  pivot_wider(id_cols = c(.chain, .iteration, .draw, month), names_from = fertilizer, values_from = contrast) %>%
  mutate(contr_HL = H - L, contr_HM = H - M, contr_ML = M - L) %>%
  select(-c(L,M,H)) %>%
  pivot_longer(starts_with('contr'), values_to = 'contrast')

ggplot(ltd_pred_wider2, aes(x = month, y = contrast)) +
  facet_wrap(~ name, nrow = 3, labeller = labeller(name = c(contr_HL = 'high - low', contr_HM = 'high - medium', contr_ML = 'medium - low'))) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  labs(y = 'Difference in inferred treatment effect (modeled)') +
  geom_hline(yintercept = 0, linetype = 'dashed', color = 'gray50') +
  ggtitle('Difference in inferred treatment effect on tip density (IRREC)', 'between pairs of fertilizer levels at different times since release')




###########Proportion of dead tips##############

ggplot(all_IRREC, aes(x = months_since_exp_start, y = percent_dead, color = fertilizer)) +
  geom_point(alpha = 1/4) +
  geom_boxplot(aes(group = interaction(fertilizer, months_since_exp_start))) +
  stat_summary(geom = 'line', fun = median) +
  geom_vline(xintercept = month_offset, size = 1.5) 


propdeadtips_monthspline_trt_fit2 <- brm(
  dead_tips | trials(total_tips) ~ fertilizer + pre_vs_post + months_since_exp_start 
  + months_since_trt_start + fertilizer:pre_vs_post + fertilizer:months_since_exp_start 
  + fertilizer:months_since_trt_start + s(month, k = 10, m = 1) + (1 | plant), 
  data = all_IRREC, family = binomial(link = 'logit'), 
  prior = c(
    prior(normal(0, 3), class = b)
  ),
  chains = 4, iter = 3000, warmup = 2000, seed = 2288,
  file = '/Users/dalehalbritter/Documents/Demography/propdeadtips_monthspline_trt2'
)


max(rhat(propdeadtips_monthspline_trt_fit2))

pp_check(propdeadtips_monthspline_trt_fit2) + scale_x_log10()



propdeadtips_predictions_real2 <- pred_grid_real2 %>%
  mutate(total_tips = 1) %>%
  add_epred_draws(propdeadtips_monthspline_trt_fit2, re_formula = ~ month) %>%
  mutate(scenario = 'real')

propdeadtips_predictions_counterfactual2 <- pred_grid_counterfactual2 %>%
  mutate(total_tips = 1) %>%
  add_epred_draws(propdeadtips_monthspline_trt_fit2, re_formula = ~ month) %>%
  mutate(scenario = 'counterfactual')

propdeadtips_predictions2 <- bind_rows(propdeadtips_predictions_real2, propdeadtips_predictions_counterfactual2) 



ggplot(propdeadtips_predictions2, aes(x = months_since_exp_start, y = .epred, fill = scenario)) +
  facet_wrap(~ fertilizer, nrow = 3, labeller = labeller(fertilizer = c(L = 'low fertilizer', M = 'medium fertilizer', H = 'high fertilizer'))) +
  stat_lineribbon(.width = c(0.95, 0.66), alpha = 1/2) +
  labs(y = 'Proportion dead tips (modeled)') +
  ylim(0,1) +
  scale_x_continuous(name = 'Months since start of experiment', expand = c(0, 0)) +
  theme(legend.position = c(0.1, 0.9), legend.background = element_blank()) +
  scale_fill_brewer(palette = 'Dark2')


pdt_pred_real2 <- propdeadtips_predictions_real2 %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_real = .epred)

pdt_pred_cf2 <- propdeadtips_predictions_counterfactual2 %>%
  ungroup %>%
  select(-c(pre_vs_post, months_since_trt_start, month, scenario, .row)) %>%
  rename(pred_cf = .epred)

pdt_pred_wide2 <- inner_join(pdt_pred_real2, pdt_pred_cf2) %>%
  mutate(contrast = pred_real/pred_cf, month = months_since_exp_start - month_offset) 


ggplot(pdt_pred_wide2, aes(x = month, y = contrast)) + 
  facet_wrap(~ fertilizer, nrow = 3, labeller = fert_label) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  scale_y_continuous(trans=scales::pseudo_log_trans(base=10), breaks=c(0,2.0e+08)) +
  labs(y = 'Ratio of proportion dead tips with thrips/without thrips (modeled)') +
  geom_hline(yintercept = 1, linetype = 'dashed', color = 'gray50') +
  ggtitle('Inferred treatment effect on proportion dead tips (IRREC)', 'by fertilizer level at different times since release')



pdt_pred_wider2 <- pdt_pred_wide2 %>%
  select(-c(months_since_exp_start)) %>%
  pivot_wider(id_cols = c(.chain, .iteration, .draw, month), names_from = fertilizer, values_from = contrast) %>%
  mutate(contr_HL = H - L, contr_HM = H - M, contr_ML = M - L) %>%
  select(-c(L,M,H)) %>%
  pivot_longer(starts_with('contr'), values_to = 'contrast')

ggplot(pdt_pred_wider2, aes(x = month, y = contrast)) +
  facet_wrap(~ name, nrow = 3, labeller = labeller(name = c(contr_HL = 'high - low', contr_HM = 'high - medium', contr_ML = 'medium - low'))) +
  stat_interval(.width = c(.66, .95, .99)) +
  scale_color_brewer(palette = 'Blues') +
  scale_x_continuous(name = 'Months since thrips release', breaks = c(6, 12, 18)) +
  scale_y_continuous(trans=scales::pseudo_log_trans(base=10), breaks=c(0,2.0e+08)) +
  labs(y = 'Difference in inferred treatment effect (modeled)') +
  geom_hline(yintercept = 0, linetype = 'dashed', color = 'gray50') +
  ggtitle('Difference in inferred treatment effect on proportion dead tips (IRREC)', 'between pairs of fertilizer levels at different times since release')




print_qi <- function(l, u, digits = 2) glue('({round(l, digits)}, {round(u, digits)})')

make_table <- function(pred_data) {
  pred_data %>%
    group_by(fertilizer, month) %>%
    median_qi(pred_real2, pred_cf, contrast, .width = c(.66, .95, .99)) %>% 
    select(-c(.point, .interval)) %>% 
    pivot_longer(-c(fertilizer, month, .width)) %>%
    separate(name, into = c('param', 'quantile'), sep = '\\.') %>%
    mutate(quantile = if_else(is.na(quantile), 'median', quantile)) %>%
    pivot_wider(names_from = quantile, values_from = value) %>%
    mutate(qi = print_qi(lower, upper)) %>%
    pivot_wider(id_cols = c(fertilizer, month, param, median), names_from = .width, values_from = qi, names_prefix = 'qi') 
}

display_table <- function(qi_data, table_title) {
  qi_data %>%
    ungroup %>%
    mutate(fertilizer = factor(fertilizer, levels = c('L', 'M', 'H', 'overall'), labels = c('low fertilizer', 'medium fertilizer', 'high fertilizer', 'overall')),
           param = rep(c('with thrips', 'without thrips (counterfactual)', 'with/without contrast'), nrow(.)/3)) %>%
    gt(groupname_col = 'fertilizer', caption = table_title) %>%
    cols_label(month = 'months since release', param = 'estimated mean', qi0.66 = '66% QI', qi0.95 = '95% QI', qi0.99 = '99% QI') %>%
    fmt_number(columns = median, decimals = 2) %>%
    tab_options(
      row_group.background.color = "#D4EBF2",
      row_group.font.weight = 'bold',
      column_labels.font.weight = 'bold'
    )
}

ltd_pred_qi <- ltd_pred_wide2 %>% mutate(pred_real2 = exp(pred_real2), pred_cf = exp(pred_cf)) %>% make_table()
ltd_pred_qi_nofert <- ltd_pred_wide_nofert %>% mutate(pred_real2 = exp(pred_real2), pred_cf = exp(pred_cf), fertilizer = 'overall') %>% make_table()
pdt_pred_qi <- make_table(pdt_pred_wide)
pdt_pred_qi_nofert <- pdt_pred_wider2 %>% mutate(fertilizer = 'overall') %>% make_table()